let options;

init();

async function init() {
	options = (await extensionStorage.get('options')).options;

	if (!options) {
		log('No extension options found. Aborting');
		return;
	}

	browser.runtime.onMessage.addListener((message, sender) => {
		if (message.type === 'optionsUpdated') {
			options = message.data;
		}
	});

	window.addEventListener('mousedown', onMouseDown, true);

	window.addEventListener('contextmenu', detectOrphanContextMenu, true);

	window.addEventListener('keydown', onKeyDown, true);
}

let withinARightClickSequence = false;

function onMouseDown(mouseDownEvent) {
	if (mouseDownEvent.button != 2) {
		return;
	}

	log("Right mouse down event triggered");

	withinARightClickSequence = true;
	let rightMouseUpTriggered = false;
	let contextMenuTriggered = false;
	let gestureDetected = false;

	const startX = mouseDownEvent.pageX; 
	const startY = mouseDownEvent.pageY;

	window.addEventListener('mouseup', onMouseUp, true);
	window.addEventListener('contextmenu', onContextMenu, true);
	window.addEventListener('click', onClick, true);

	async function onMouseUp(event) {
		if (event.button != 2) {
			return;
		}

		log("Right mouse up event triggered.");
		window.removeEventListener('mouseup', onMouseUp, true);

		const contextMenuTriggeredBeforeMouseUp = contextMenuTriggered;
		rightMouseUpTriggered = true;

		const deltaX = event.pageX - startX;
		const absDeltaX = Math.abs(deltaX);

		const deltaY = event.pageY - startY;
		const absDeltaY = Math.abs(deltaY);

		log(`Delta X: ${deltaX}, Delta Y: ${deltaY}`)
        		
		const minDelta = options.minDelta;
		gestureDetected = absDeltaX >= minDelta || absDeltaY >= minDelta;

		if (gestureDetected) {
			event.preventDefault();
			event.stopImmediatePropagation();
            			
			const deltaYToXRatio = absDeltaY / absDeltaX;

			if (deltaYToXRatio <= 1) {
				if (options.leftEnabled && deltaX < minDelta * -1) {
					await handleGesture('left', event.ctrlKey);
				} else if (options.rightEnabled && deltaX >= minDelta) {
					await handleGesture('right', event.ctrlKey);
				}
			} else {
				if (options.upEnabled && deltaY < minDelta * -1) {
					await handleGesture('up', event.ctrlKey);
				} else if (options.downEnabled && deltaY >= minDelta) {
					await handleGesture('down', event.ctrlKey);
				}
			}
		} else if (contextMenuTriggeredBeforeMouseUp) {
				log('Context menu was suppressed as it triggered before mouseUp but eventually no gesture was detected')
		}

		if (contextMenuTriggeredBeforeMouseUp) {
			withinARightClickSequence = false;
		}
	}

	function onContextMenu(event) {
		window.removeEventListener('contextmenu', onContextMenu, true);
		contextMenuTriggered = true;

		log("Context menu event triggered.")

		if (!rightMouseUpTriggered) {
			log("Context menu suppressed as mouse up event has not yet fired");
			event.preventDefault();
			event.stopImmediatePropagation();
		} else if (gestureDetected) {
			log("Context menu suppressed as a gesture has been detected");
			event.preventDefault();
			event.stopImmediatePropagation();
		}

		if (rightMouseUpTriggered) {
			withinARightClickSequence = false;
		}
	}

	function onClick(event) {
		if (event.button != 2) {
			return
		}

		window.removeEventListener('click', onClick, true);

		log("Right click event triggered");

		if (gestureDetected) {
			log("Right click event suppressed as a gesture has been detected");
			event.preventDefault();
			event.stopImmediatePropagation();
		}
	}
}

let lastKeyPressed = -1;

async function onKeyDown(event) {
	lastKeyPressed = event.keyCode;
}

function detectOrphanContextMenu(event) {
	if (!withinARightClickSequence) {
		if (lastKeyPressed == 93) {
			lastKeyPressed = -1;
			return;
		}

		log("Orphan context menu event suppressed");
		event.preventDefault();
		event.stopImmediatePropagation();
	}
}

async function handleGesture(gesture, ctrlKeyPressed) {
	switch (gesture) {
		case "left":
			if (ctrlKeyPressed) {
				if (options.ctrlLeftEnabled) {
					await executeAction(options.ctrlLeftAction);
				}
			} else {
				if (options.leftEnabled) {
					await executeAction(options.leftAction);
				}
			}
			break;

		case "right":
			if (ctrlKeyPressed) {
				if (options.ctrlRightEnabled) {
					await executeAction(options.ctrlRightAction);
				}
			} else {
				if (options.rightEnabled) {
					await executeAction(options.rightAction);
				}
			}

			break;

		case "up":
			if (ctrlKeyPressed) {
				if (options.ctrlUpEnabled) {
					await executeAction(options.ctrlUpAction);
				}
			} else {
				if (options.upEnabled) {
					await executeAction(options.upAction);
				}
			}
			break;

		case "down":
			if (ctrlKeyPressed) {
				if (options.ctrlDownEnabled) {
					await executeAction(options.ctrlDownAction);
				}
			} else {
				if (options.downEnabled) {
					await executeAction(options.downAction);
				}
			}
			break;
	}
}

async function executeAction(action) {
	switch (action) {
		case "navigateBack":
			window.history.back();
			break;

		case "navigateForward":
			window.history.forward();
			break;

		case "reload":
			if (window.self === window.top) {
				window.location.reload();
			} else {
				await sendRequest('reloadCurrentTab');
			}
			break;

		case "closeCurrentTab":
			await sendRequest('closeCurrentTab');
			break;

		case "createNewTab":
			await sendRequest('createNewTab');
			break;

		case "activateLeftTab":
			await sendRequest('activateLeftTab', { skipTabsWithSpecialPages: false });
			break;

		case "activateLeftTabSkipSpecial":
			await sendRequest('activateLeftTab', { skipTabsWithSpecialPages: true });
			break;

		case "activateRightTab":
			await sendRequest('activateRightTab', { skipTabsWithSpecialPages: false });
			break;

		case "activateRightTabSkipSpecial":
			await sendRequest('activateRightTab', { skipTabsWithSpecialPages: true });
			break;

		case "undoCloseTab":
			await sendRequest('undoCloseTab');
			break;
	}
}